@@ -129,15 +129,27 @@ class Minuit:
     @property
     def tol(self) -> float:
         """
-        Access tolerance for convergence.
+        Access tolerance for convergence with the EDM criterion.
 
-        The main convergence criteria of MINUIT is `edm < edm_max`, where `edm_max`
-        is calculated as `edm_max = 0.002 * tol * errordef` in case of the MIGRAD
-        algorithm and as `edm_max = tol * errordef` in case of the SIMPLEX algorithm.
-        EDM stands for *Estimated Distance to Minimum*, which is described in the
-        `MINUIT paper`_. The EDM criterion is well suited for statistical cost functions,
+        Minuit detects converge with the EDM criterion. EDM stands for *Estimated
+        Distance to Minimum*, it is mathematically described in the `MINUIT paper`_.
+        The EDM criterion is well suited for statistical cost functions,
         since it stops the minimization when parameter improvements become small
         compared to parameter uncertainties.
+
+        The convergence is detected when `edm < edm_max`, where `edm_max` is calculated
+        as
+
+            * Migrad: edm_max = 0.002 * tol * errordef
+            * Simplex: edm_max = tol * errordef
+
+        Users can set `tol` (default: 0.1) to a different value to speed up convergence
+        at the cost of a larger error on the fitted parameters and possibly invalid
+        estimates for parameter uncertainties.
+
+        Under some circumstances, Migrad is allowed to violate edm_max by a factor of
+        10. Users should not try to detect convergence by comparing edm with edm_max,
+        but query :attr:`iminuit.util.FMin.is_above_max_edm`.
         """
         return self._tolerance
 
@@ -279,8 +291,8 @@ class Minuit:
         In case of complex fits, it can help to limit some parameters first, run Migrad,
         then remove the limits and run Migrad again. Limits will bias the result only if
         the best fit value is outside the limits, not if it is inside. Limits will affect
-        the estimated HESSE uncertainties if the parameter is close to a limit. They do
-        not affect the MINOS uncertainties, because those are invariant to
+        the estimated Hesse uncertainties if the parameter is close to a limit. They do
+        not affect the Minos uncertainties, because those are invariant to
         transformations and limits are implemented via a variable transformation.
 
         See Also
@@ -720,26 +732,34 @@ class Minuit:
 
         Notes
         -----
-        Originally this was supposed to use MnScan from C++ Minuit2, but MnScan is broken.
-        It does a 1D scan with 41 steps for each parameter in sequence, so it is not
-        actually scanning the full hypercube. It first scans one parameter, then starts
-        the scan of the second parameter from the best value of the first and so on.
-        Other issues: One cannot configure the number of steps. A gradient and second
-        derivatives are computed for the starting values only to be discarded.
+        The scan can return an invalid minimum, this is not a cause for alarm. It just
+        minimizes the cost function, the EDM value is only computed after the scan found
+        a best point. If the best point still has a bad EDM value, the minimum is
+        considered invalid. But even if it is considered valid, it is probably not
+        accurate, since the tolerance is very lax. One should always run :meth:`migrad`
+        after the scan.
 
-        This implementation here does a full scan of the hypercube in Python. Returning a
-        valid FunctionMinimum object was a major challenge, because C++ Minuit2 does not
-        allow one to initialize data objects with data, it forces one to go through
-        algorithm objects. Because of that design, the Minuit2 C++ interface forces one to
-        compute the gradient and second derivatives for the starting values, even though
-        these are not used in a scan. We turn a disadvantage into an advantage by tricking
-        Minuit2 into computing updates of the step sizes and to estimate the EDM value.
+        This implementation here does a full scan of the hypercube in Python. Originally,
+        this was supposed to use MnScan from C++ Minuit2, but MnScan is unsuitable.
+        It does a 1D scan with 41 steps (not configurable) for each parameter in sequence,
+        so it is not actually scanning the full hypercube. It first scans one parameter,
+        then starts the scan of the second parameter from the best value of the first and
+        so on. This fails easily when the parameters are correlated.
         """
+        # Implementation notes:
+        # Returning a valid FunctionMinimum object was a major challenge, because C++
+        # Minuit2 does not allow one to initialize data objects with data, it forces one
+        # to go through algorithm objects. Because of that design, the Minuit2 C++
+        # interface forces one to compute the gradient and second derivatives for the
+        # starting values, even though these are not used in a scan. We turn a
+        # disadvantage into an advantage here by tricking Minuit2 into computing updates
+        # of the step sizes and to estimate the EDM value.
+
         # Running MnScan would look like this:
-        # scan = MnScan(self._fcn, self._last_state, self.strategy)
-        # fm = scan(0, 0)  # args are ignored
-        # self._last_state = fm.state
-        # self._fmin = mutil.FMin(fm, self._fcn.nfcn, self._fcn.ngrad, self._tolerance)
+        #  scan = MnScan(self._fcn, self._last_state, self.strategy)
+        #  fm = scan(0, 0)  # args are ignored
+        #  self._last_state = fm.state
+        #  self._fmin = mutil.FMin(fm, self._fcn.nfcn, self._fcn.ngrad, self._tolerance)
 
         n = self.nfit
         if ncall is None:
@@ -793,24 +813,33 @@ class Minuit:
 
     def hesse(self, ncall: Optional[int] = None):
         r"""
-        Run HESSE algorithm to compute asymptotic errors.
+        Run Hesse algorithm to compute asymptotic errors.
 
-        HESSE estimates the covariance matrix by inverting the matrix of
+        The Hesse method estimates the covariance matrix by inverting the matrix of
         `second derivatives (Hesse matrix) at the minimum
-        <http://en.wikipedia.org/wiki/Hessian_matrix>`_. This covariance matrix is valid
-        if your :math:`\chi^2` or likelihood profile looks like a hyperparabola around
-        the minimum. Asymptotically (large samples) this is always the case, but in small
-        samples the uncertainty estimate is approximate. If you want to know how your
-        parameters are correlated, you also need to use this. The MINOS algorithm is
-        another way to estimate parameter uncertainties, see :meth:`minos`.
+        <http://en.wikipedia.org/wiki/Hessian_matrix>`_. To get parameters correlations,
+        you need to use this. The Minos algorithm is another way to estimate parameter
+        uncertainties, see :meth:`minos`.
 
         Parameters
         ----------
         ncall :
-            Approximate upper limit for the number of calls made by the HESSE algorithm.
+            Approximate upper limit for the number of calls made by the Hesse algorithm.
             If set to None, use the adaptive heuristic from the Minuit2 library
             (Default: None).
 
+        Notes
+        -----
+        The covariance matrix is asymptotically (in large samples) valid. By valid we mean
+        that confidence intervals constructed from the errors contain the true value with
+        a well-known coverage probability (68 % for each interval). In finite samples,
+        this is likely to be true if your cost function looks like a hyperparabola around
+        the minimum.
+
+        In practice, the errors very likely have correct coverage if the results from
+        Minos and Hesse methods agree. It is possible to construct artifical functions
+        where this rule is violated, but in practice it should always work.
+
         See Also
         --------
         minos
@@ -821,7 +850,7 @@ class Minuit:
         # parameters are fixed
         if self.nfit == 0:
             warn(
-                "HESSE called with all parameters fixed",
+                "Hesse called with all parameters fixed",
                 mutil.IMinuitWarning,
                 stacklevel=2,
             )
@@ -843,7 +872,7 @@ class Minuit:
 
         if self._last_state.has_covariance is False:
             if not self._fmin:
-                raise RuntimeError("HESSE Failed")
+                raise RuntimeError("Hesse Failed")
 
         self._make_covariance()
 
@@ -1003,7 +1032,7 @@ class Minuit:
 
         x = np.linspace(bound[0], bound[1], size, dtype=np.double)
         y = np.empty(size, dtype=np.double)
-        status = np.empty(size, dtype=np.bool)
+        status = np.empty(size, dtype=bool)
 
         state = MnUserParameterState(self._last_state)  # copy
         ipar = self._var2pos[vname]
@@ -1300,7 +1329,7 @@ class Minuit:
         self, x: str, y: str, *, cl: Optional[float] = None, size: int = 100
     ) -> Sequence[Sequence[float]]:
         """
-        Get 2D MINOS confidence region.
+        Get 2D Minos confidence region.
 
         This scans over two parameters and minimises all other free parameters for each
         scan point. This scan produces a statistical confidence region according to the
