@@ -1,17 +1,19 @@
-"""iminuit utility functions and classes."""
+"""Data classes and utilities used by :class:`iminuit.Minuit`."""
 import inspect
 from collections import OrderedDict
 from argparse import Namespace
-from collections.abc import Iterable
 from . import _repr_html
 from . import _repr_text
 import numpy as np
+from typing import Dict, Sequence, Iterable
+import types
+import warnings
 
 inf = float("infinity")
 
 
 class IMinuitWarning(RuntimeWarning):
-    """iminuit warning."""
+    """Generic iminuit warning."""
 
 
 class HesseFailedWarning(IMinuitWarning):
@@ -19,57 +21,76 @@ class HesseFailedWarning(IMinuitWarning):
 
 
 class BasicView:
-    """Array-like view of parameter state.
+    """
+    Array-like view of parameter state.
 
     Derived classes need to implement methods _set and _get to access
-    specific properties of the parameter state."""
+    specific properties of the parameter state.
+    """
 
     __slots__ = ("_minuit", "_ndim")
 
     def __init__(self, minuit, ndim=0):
+        """Not to be initialized by users."""
         self._minuit = minuit
         self._ndim = ndim
 
     def __iter__(self):
+        """Get iterator over values."""
         for i in range(len(self)):
             yield self._get(i)
 
     def __len__(self):
+        """Get number of paramters."""
         return self._minuit.npar
 
     def __getitem__(self, key):
-        if isinstance(key, slice):
-            ind = range(*key.indices(len(self)))
-            return [self._get(i) for i in ind]
-        i = _key2index(self._minuit._var2pos, key)
-        return self._get(i)
+        """
+        Get values of the view.
+
+        Parameters
+        ----------
+        key: int, str, slice, list of int or str
+            If the key is an int or str, return corresponding value.
+            If it is a slice, list of int or str, return the corresponding subset.
+        """
+        key = _key2index(self._minuit._var2pos, key)
+        if isinstance(key, list):
+            return [self._get(i) for i in key]
+        return self._get(key)
 
     def __setitem__(self, key, value):
+        """Assign a new value at key, which can be an index, a parameter name, or a slice."""
         self._minuit._copy_state_if_needed()
-        if isinstance(key, slice):
-            ind = range(*key.indices(len(self)))
-            if _ndim(value) == self._ndim:  # basic broadcasting
-                for i in ind:
+        key = _key2index(self._minuit._var2pos, key)
+        if isinstance(key, list):
+            if _ndim(value) == self._ndim:  # support basic broadcasting
+                for i in key:
                     self._set(i, value)
             else:
-                if len(value) != len(ind):
+                if len(value) != len(key):
                     raise ValueError("length of argument does not match slice")
-                for i, v in zip(ind, value):
+                for i, v in zip(key, value):
                     self._set(i, v)
         else:
-            i = _key2index(self._minuit._var2pos, key)
-            self._set(i, value)
+            self._set(key, value)
 
     def __eq__(self, other):
+        """Return true if all values are equal."""
         return len(self) == len(other) and all(x == y for x, y in zip(self, other))
 
     def __repr__(self):
+        """Get detailed text representation."""
         s = f"<{self.__class__.__name__}"
         for (k, v) in zip(self._minuit._pos2var, self):
             s += f" {k}={v}"
         s += ">"
         return s
 
+    def to_dict(self) -> Dict[str, float]:
+        """Obtain dict representation."""
+        return {k: self._get(i) for i, k in enumerate(self._minuit._pos2var)}
+
 
 def _ndim(obj):
     nd = 0
@@ -120,6 +141,10 @@ class FixedView(BasicView):
 class LimitView(BasicView):
     """Array-like view of parameter limits."""
 
+    def __init__(self, minuit):
+        """Not to be initialized by users."""
+        super(LimitView, self).__init__(minuit, 1)
+
     def _get(self, i):
         p = self._minuit._last_state[i]
         return (
@@ -176,6 +201,7 @@ class Matrix(np.ndarray):
     __slots__ = ("_var2pos",)
 
     def __new__(cls, parameters):
+        """Not to be initialized by users."""
         if isinstance(parameters, dict):
             var2pos = parameters
         elif isinstance(parameters, tuple):
@@ -188,24 +214,37 @@ class Matrix(np.ndarray):
         return obj
 
     def __array_finalize__(self, obj):
+        """For internal use."""
         if obj is None:
-            return
-
-        self._var2pos = getattr(obj, "_var2pos")
+            self._var2pos = None
+        else:
+            self._var2pos = getattr(obj, "_var2pos", None)
 
     def __getitem__(self, key):
-        var2pos = self._var2pos
-        if isinstance(key, tuple):
-            key = tuple((k if isinstance(k, int) else var2pos[k]) for k in key)
+        """Get matrix element at key."""
+        var2pos = self._var2pos or {}
+        if isinstance(key, tuple):  # tuple is special case for __getitem__
+            key = tuple(var2pos.get(k, k) for k in key)
         elif isinstance(key, str):
             key = var2pos[key]
+        elif isinstance(key, Sequence):
+            key = list(var2pos.get(k, k) for k in key)
+            t = super(Matrix, self).__getitem__(key).T
+            return super(Matrix, t).__getitem__(key).T
+        elif isinstance(key, slice):
+            key = slice(var2pos.get(key.start), var2pos.get(key.stop), key.step)
+            return super(Matrix, self).__getitem__((key, key))
         return super(Matrix, self).__getitem__(key)
 
     def to_table(self):
         """
-        Converts the matrix to a tabular format consumable by the external
+        Convert matrix to tabular format.
+
+        The output is consumable by the external
         `tabulate <https://pypi.org/project/tabulate>`_ module.
 
+        Examples
+        --------
         >>> import tabulate as tab
         >>> from iminuit import Minuit
         >>> m = Minuit(lambda x, y: x ** 2 + y ** 2, x=1, y=2).migrad()
@@ -225,7 +264,7 @@ class Matrix(np.ndarray):
 
     def correlation(self):
         """
-        Computes and returns the correlation matrix.
+        Compute and return correlation matrix.
 
         If the matrix is already a correlation matrix, this effectively returns a copy
         of the original matrix.
@@ -236,10 +275,12 @@ class Matrix(np.ndarray):
         return a
 
     def __repr__(self):
+        """Get detailed text representation."""
         return super(Matrix, self).__str__()
 
     def __str__(self):
-        return repr(self)
+        """Get user-friendly text representation."""
+        return _repr_text.matrix(self)
 
     def _repr_html_(self):
         return _repr_html.matrix(self)
@@ -248,11 +289,30 @@ class Matrix(np.ndarray):
         if cycle:
             p.text("<Matrix ...>")
         else:
-            p.text(_repr_text.matrix(self))
+            p.text(str(self))
 
 
 class FMin:
-    """Function minimum view."""
+    """
+    Function minimum view.
+
+    This object provides detailed metadata about the function minimum. Inspect this to
+    check what exactly happened if the fit did not converge. Use the
+    :class:`iminuit.Minuit` object to get the best fit values, their
+    uncertainties, or the function value at the minimum. For convenience, you can also
+    get a basic OK from :class:`iminuit.Minuit` with the methods
+    :attr:`iminuit.Minuit.valid` and :attr:`iminuit.Minuit.accurate`.
+
+    See Also
+    --------
+    :attr:`iminuit.Minuit.values`
+    :attr:`iminuit.Minuit.errors`
+    :attr:`iminuit.Minuit.merrors`
+    :attr:`iminuit.Minuit.covariance`
+    :attr:`iminuit.Minuit.fval`
+    :attr:`iminuit.Minuit.valid`
+    :attr:`iminuit.Minuit.accurate`
+    """
 
     __slots__ = (
         "_src",
@@ -263,6 +323,7 @@ class FMin:
     )
 
     def __init__(self, fmin, nfcn, ngrad, edm_goal):
+        """Not to be initialized by users."""
         self._src = fmin
         self._has_parameters_at_limit = False
         for mp in fmin.state:
@@ -280,7 +341,8 @@ class FMin:
 
     @property
     def edm(self):
-        """Estimated Distance to Minimum.
+        """
+        Get Estimated Distance to Minimum.
 
         Minuit uses this criterion to determine whether the fit converged. It depends
         on the gradient and the Hessian matrix. It measures how well the current
@@ -292,18 +354,22 @@ class FMin:
 
     @property
     def edm_goal(self):
-        """EDM threshold value for stopping the minimization. The threshold is allowed
-        to be violated up to a factor of 10 in some situations."""
+        """
+        Get EDM threshold value for stopping the minimization.
+
+        The threshold is allowed to be violated by a factor of 10 in some situations.
+        """
         return self._edm_goal
 
     @property
     def fval(self):
-        """Value of the cost function at the minimum."""
+        """Get cost function value at the minimum."""
         return self._src.fval
 
     @property
     def has_parameters_at_limit(self):
-        """Whether any bounded parameter was fitted close to a bound.
+        """
+        Return whether any bounded parameter was fitted close to a bound.
 
         The estimated error for the affected parameters is usually off. May be an
         indication to remove or loosen the limits on the affected parameter.
@@ -312,23 +378,24 @@ class FMin:
 
     @property
     def nfcn(self):
-        """Number of function calls so far."""
+        """Get number of function calls so far."""
         return self._nfcn
 
     @property
     def ngrad(self):
-        """Number of function gradient calls so far."""
+        """Get number of function gradient calls so far."""
         return self._ngrad
 
     @property
     def is_valid(self):
-        """Whether Migrad converged successfully.
+        """
+        Return whether Migrad converged successfully.
 
         For it to return True, the following conditions need to be fulfilled:
 
-          - has_valid_parameters is True
-          - has_reached_call_limit is False
-          - is_above_max_edm is False
+          - :attr:`has_valid_parameters` is True
+          - :attr:`has_reached_call_limit` is False
+          - :attr:`is_above_max_edm` is False
 
         Note: The actual verdict is computed inside the Minuit2 C++ code, so we
         cannot guarantee that is_valid is exactly equivalent to these conditions.
@@ -337,12 +404,13 @@ class FMin:
 
     @property
     def has_valid_parameters(self):
-        """Whether parameters are valid.
+        """
+        Return whether parameters are valid.
 
         For it to return True, the following conditions need to be fulfilled:
 
-          - has_reached_call_limit is False
-          - is_above_max_edm is False
+          - :attr:`has_reached_call_limit` is False
+          - :attr:`is_above_max_edm` is False
 
         Note: The actual verdict is computed inside the Minuit2 C++ code, so we
         cannot guarantee that is_valid is exactly equivalent to these conditions.
@@ -351,50 +419,71 @@ class FMin:
 
     @property
     def has_accurate_covar(self):
-        """Whether the covariance matrix is accurate.
+        """
+        Return whether the covariance matrix is accurate.
 
         While Migrad runs, it computes an approximation to the current Hessian
         matrix. If the strategy is set to 0 or if the fit did not converge, the
         inverse of this approximation is returned instead of the inverse of the
         accurately computed Hessian matrix. This property returns False if the
-        approximation has been returned instead of an accurate matrix.
+        approximation has been returned instead of an accurate matrix computed by
+        the Hesse method.
         """
         return self._src.has_accurate_covar
 
     @property
     def has_posdef_covar(self):
-        """Whether the Hessian matrix is positive definite.
+        """
+        Return whether the Hessian matrix is positive definite.
 
-        This must be the case if the extremum is a minimum. Otherwise it is a
-        maximum or a saddle point.
+        This must be the case if the extremum is a minimum, otherwise it is a saddle
+        point. If it returns False, the fitted result may be correct, but the reported
+        uncertainties are false. This may affect some parameters or all of them.
+        Possible causes:
 
-        If the fit has converged, this should always be true. It may be false if the
-        fit did not converge or was stopped prematurely.
+            * Model contains redundanted parameters that are 100% correlated. Fix:
+              remove the parameters that are 100% correlated.
+            * Cost function is not computed in double precision. Fix: try adjusting
+              :attr:`iminuit.Minuit.precision` or change the cost function to compute
+              in double precision.
+            * Cost function is not analytical near the minimum. Fix: change the cost
+              function to something analytical. Functions are not analytical if:
+
+                * It does computations based on (pseudo)random numbers.
+                * It contains vertical steps, for example from code like this::
+
+                      if cond:
+                          return value1
+                      else:
+                          return value2
         """
         return self._src.has_posdef_covar
 
     @property
     def has_made_posdef_covar(self):
-        """Whether the matrix was forced to be positive definite.
+        """
+        Return whether the matrix was forced to be positive definite.
 
         While Migrad runs, it computes an approximation to the current Hessian matrix.
         It can happen that this approximation is not positive definite, but that is
         required to compute the next Newton step. Migrad then adds an appropriate
         diagonal matrix to enforce positive definiteness.
 
-        If the fit has converged, this should always be false. It may be true if the
-        fit did not converge or was stopped prematurely.
+        If the fit has converged successfully, this should always return False. If
+        Minuit forced the matrix to be positive definite, the parameter uncertainties
+        are false, see :attr:`has_posdef_covar` for more details.
         """
         return self._src.has_made_posdef_covar
 
     @property
     def hesse_failed(self):
-        """Whether the last call to Hesse failed."""
+        """Return whether the last call to Hesse failed."""
         return self._src.hesse_failed
 
     @property
     def has_covariance(self):
-        """Whether a covariance matrix was computed at all.
+        """
+        Return whether a covariance matrix was computed at all.
 
         This is false if the Simplex minimization algorithm was used instead of
         Migrad, in which no approximation to the Hessian is computed.
@@ -403,29 +492,34 @@ class FMin:
 
     @property
     def is_above_max_edm(self):
-        """Whether the EDM value is below the convergence threshold.
+        """
+        Return whether the EDM value is below the convergence threshold.
 
-        If this is true, the fit did not converge; otherwise this is false.
+        Returns True, if the fit did not converge; otherwise returns False.
         """
         return self._src.is_above_max_edm
 
     @property
     def has_reached_call_limit(self):
-        """Whether Migrad exceeded the allowed number of function calls.
+        """
+        Return whether Migrad exceeded the allowed number of function calls.
 
-        If this is true, the fit was stopped before convergence was reached.
+        Returns True true, the fit was stopped before convergence was reached;
+        otherwise returns False.
         """
         return self._src.has_reached_call_limit
 
     @property
     def errordef(self):
-        """Equal to the value of ``Minuit.errordef`` when Migrad ran."""
+        """Equal to the value of :attr:`iminuit.Minuit.errordef` when Migrad ran."""
         return self._src.errordef
 
     def __eq__(self, other):
+        """Return True if all attributes are equal."""
         return all(getattr(self, k) == getattr(other, k) for k in self.__slots__)
 
     def __repr__(self):
+        """Get detailed text representation."""
         s = "<FMin"
         for key in sorted(dir(self)):
             if key.startswith("_"):
@@ -435,6 +529,10 @@ class FMin:
         s += ">"
         return s
 
+    def __str__(self):
+        """Get user-friendly text representation."""
+        return _repr_text.fmin(self)
+
     def _repr_html_(self):
         return _repr_html.fmin(self)
 
@@ -442,7 +540,7 @@ class FMin:
         if cycle:
             p.text("<FMin ...>")
         else:
-            p.text(_repr_text.fmin(self))
+            p.text(str(self))
 
 
 class Param:
@@ -464,25 +562,32 @@ class Param:
     )
 
     def __init__(self, *args):
+        """Not to be initialized by users."""
         assert len(args) == len(self.__slots__)
         for k, arg in zip(self.__slots__, args):
             setattr(self, k, arg)
 
     def __eq__(self, other):
+        """Return True if all values are equal."""
         return all(getattr(self, k) == getattr(other, k) for k in self.__slots__)
 
     def __repr__(self):
+        """Get detailed text representation."""
         pairs = []
         for k in self.__slots__:
             v = getattr(self, k)
             pairs.append(f"{k}={v!r}")
         return "Param(" + ", ".join(pairs) + ")"
 
+    def __str__(self):
+        """Get user-friendly text representation."""
+        return _repr_text.params([self])
+
     def _repr_pretty_(self, p, cycle):
         if cycle:
             p.text("Param(...)")
         else:
-            p.text(_repr_text.params([self]))
+            p.text(str(self))
 
 
 class Params(tuple):
@@ -495,9 +600,13 @@ class Params(tuple):
 
     def to_table(self):
         """
-        Converts parameter data to a tabular format consumable by the external
+        Convert parameter data to a tabular format.
+
+        The output is consumable by the external
         `tabulate <https://pypi.org/project/tabulate>`_ module.
 
+        Examples
+        --------
         >>> import tabulate as tab
         >>> from iminuit import Minuit
         >>> m = Minuit(lambda x, y: x ** 2 + (y / 2) ** 2 + 1, x=0, y=0)
@@ -544,6 +653,7 @@ class Params(tuple):
         return tab, header
 
     def __getitem__(self, key):
+        """Get item at key, which can be an index or a parameter name."""
         if isinstance(key, str):
             for i, p in enumerate(self):
                 if p.name == key:
@@ -551,33 +661,52 @@ class Params(tuple):
             key = i
         return super(Params, self).__getitem__(key)
 
+    def __str__(self):
+        """Get user-friendly text representation."""
+        return _repr_text.params(self)
+
     def _repr_pretty_(self, p, cycle):
         if cycle:
             p.text("Params(...)")
         else:
-            p.text(_repr_text.params(self))
+            p.text(str(self))
 
 
 class MError:
     """Minos data object.
 
-    **Attributes:**
-
-    - number: parameter index
-    - name: parameter name
-    - lower: lower error
-    - upper: upper error
-    - is_valid: whether Minos computation was successful
-    - lower_valid: whether downward scan was successful
-    - upper_valid: whether upward scan was successful
-    - at_lower_limit: whether scan reached lower limit
-    - at_upper_limit: whether scan reached upper limit
-    - at_lower_max_fcn: whether allowed number of function evaluations was exhausted
-    - at_upper_max_fcn: whether allowed number of function evaluations was exhausted
-    - lower_new_min: parameter value for new minimum, if one was found in downward scan
-    - upper_new_min: parameter value for new minimum, if one was found in upward scan
-    - nfcn: number of function calls
-    - min: function value at the new minimum
+    Attributes
+    ----------
+    number : int
+        Parameter index.
+    name : str
+        Parameter name.
+    lower : float
+        Lower error.
+    upper : float
+        Upper error.
+    is_valid : bool
+        Whether Minos computation was successful.
+    lower_valid : bool
+        Whether downward scan was successful.
+    upper_valid : bool
+        Whether upward scan was successful.
+    at_lower_limit : bool
+        Whether scan reached lower limit.
+    at_upper_limit : bool
+        Whether scan reached upper limit.
+    at_lower_max_fcn : bool
+        Whether allowed number of function evaluations was exhausted.
+    at_upper_max_fcn : bool
+        Whether allowed number of function evaluations was exhausted.
+    lower_new_min : float
+        Parameter value for new minimum, if one was found in downward scan.
+    upper_new_min : float
+        Parameter value for new minimum, if one was found in upward scan.
+    nfcn : int
+        Number of function calls.
+    min : float
+        Function value at the new minimum.
     """
 
     __slots__ = (
@@ -599,14 +728,17 @@ class MError:
     )
 
     def __init__(self, *args):
+        """Not to be initialized by users."""
         assert len(args) == len(self.__slots__)
         for k, arg in zip(self.__slots__, args):
             setattr(self, k, arg)
 
     def __eq__(self, other):
+        """Return True if all values are equal."""
         return all(getattr(self, k) == getattr(other, k) for k in self.__slots__)
 
     def __repr__(self):
+        """Get detailed text representation."""
         s = "<MError"
         for idx, k in enumerate(self.__slots__):
             v = getattr(self, k)
@@ -614,6 +746,10 @@ class MError:
         s += ">"
         return s
 
+    def __str__(self):
+        """Get user-friendly text representation."""
+        return _repr_text.merrors({None: self})
+
     def _repr_html_(self):
         return _repr_html.merrors({None: self})
 
@@ -621,7 +757,7 @@ class MError:
         if cycle:
             p.text("<MError ...>")
         else:
-            p.text(_repr_text.merrors({None: self}))
+            p.text(str(self))
 
 
 class MErrors(OrderedDict):
@@ -633,15 +769,21 @@ class MErrors(OrderedDict):
         return _repr_html.merrors(self)
 
     def __repr__(self):
+        """Get detailed text representation."""
         return "<MErrors\n  " + ",\n  ".join(repr(x) for x in self.values()) + "\n>"
 
+    def __str__(self):
+        """Get user-friendly text representation."""
+        return _repr_text.merrors(self)
+
     def _repr_pretty_(self, p, cycle):
         if cycle:
             p.text("<MErrors ...>")
         else:
-            p.text(_repr_text.merrors(self))
+            p.text(str(self))
 
     def __getitem__(self, key):
+        """Get item at key, which can be an index or a parameter name."""
         if isinstance(key, int):
             if key < 0:
                 key += len(self)
@@ -654,36 +796,206 @@ class MErrors(OrderedDict):
         return OrderedDict.__getitem__(self, key)
 
 
-def make_func_code(params):
-    """Make a func_code object to fake function signature.
+def _jacobi(fn, x, dx, tol):
+    assert np.ndim(x) == 1
+    assert np.ndim(dx) == 1
+    assert np.all(dx >= 0)
+    assert tol > 0
+    y = fn(x)
+    yrank = np.ndim(y)
+    jac = np.zeros((1 if yrank == 0 else len(y), len(x)))
+    h = np.zeros(len(x))
+    for i, hi in enumerate(dx):
+        if i > 0:
+            h[i - 1] = 0
+        if hi == 0:
+            continue
+        h[i] = hi
+        prev_esq = inf
+        while h[i] > 0:
+            yu = fn(x + h)
+            yd = fn(x - h)
+            du = (yu - y) / h[i]
+            dd = (y - yd) / h[i]
+            d = 0.5 * (du + dd)
+            delta = du - dd
+            if np.all(np.abs(delta) <= tol * np.abs(d)):
+                jac[:, i] = d
+                break
+            esq = np.dot(delta, delta)
+            if esq > prev_esq:
+                warnings.warn("no convergence, Jacobi matrix may be inaccurate")
+                # uses previous jac[:, i]
+                break
+            jac[:, i] = d
+            prev_esq = esq
+            h[i] *= 0.1
+    return y, jac
 
-    You can make a funccode from describable object by::
 
-        make_func_code(["x", "y"])
+def propagate(fn, x, cov):
     """
-    return Namespace(co_varnames=params, co_argcount=len(params))
+    Numerically propagates the covariance into a new space.
+
+    The function does error propagation. It computes C' = J C J^T, where C is the
+    covariance matrix of the input, C' the matrix of the output, and J is the Jacobi
+    matrix of first derivatives of the mapping function fn. The Jacobi matrix is
+    computed numerically.
+
+    Parameters
+    ----------
+    fn: callable
+        Vectorized function that computes y = fn(x).
+    x: array-like with shape (N,)
+        Input vector.
+    cov: array-like with shape (N, N)
+        Covariance matrix of input vector.
+
+    Returns
+    -------
+    y, ycov
+        y is the result of fn(x)
+        ycov is the propagated covariance matrix.
+    """
+    x = np.atleast_1d(x)
+    cov = np.atleast_2d(cov)
+    tol = 1e-2
+    dx = (np.diag(cov) * tol) ** 0.5
+    y, jac = _jacobi(fn, x, dx, tol)
+    ycov = np.einsum("ij,kl,jl", jac, jac, cov)
+    return y, np.squeeze(ycov) if np.ndim(y) == 0 else ycov
+
+
+def make_func_code(params):
+    """
+    Make a func_code object to fake a function signature.
+
+    Example::
+
+        def f(a, b): ...
+
+        f.func_code = make_func_code(["x", "y"])
+    """
+    return Namespace(co_varnames=tuple(params), co_argcount=len(params))
+
+
+def make_with_signature(callable, *varnames, **replacements):
+    """
+    Return new callable with altered signature.
+
+    Parameters
+    ----------
+    *varnames: sequence of str
+        Replace the first N argument names with these.
+    **replacements: mapping of str to str
+        Replace old argument name (key) with new argument name (value).
+
+    Returns
+    -------
+    callable with new argument names.
+    """
+    if replacements:
+        new_varnames = varnames
+        varnames = describe(callable)
+        if varnames:
+            n = len(new_varnames)
+            if n > len(varnames):
+                raise ValueError("varnames longer than original signature")
+            varnames[:n] = new_varnames
+        for k, v in replacements.items():
+            varnames[varnames.index(k)] = v
+        varnames = tuple(varnames)
+
+    if hasattr(callable, "__code__"):
+        c = callable.__code__
+        if c.co_argcount != len(varnames):
+            raise ValueError("number of parameters do not match")
+        if hasattr(c, "replace"):  # this was added after 3.6
+            # calling this introduces no overhead compared to original function
+            code = c.replace(co_varnames=varnames)
+            return types.FunctionType(code, globals())
+
+    # fallback implementation with additional overhead
+    class Caller:
+        def __init__(self, varnames):
+            self.func_code = make_func_code(varnames)
+
+        def __call__(self, *args):
+            return callable(*args)
+
+    return Caller(varnames)
+
+
+def merge_signatures(callables):
+    """
+    Merge signatures of callables with positional arguments.
+
+    This is best explained by an example::
+
+        def f(x, y, z): ...
+
+        def g(x, p): ...
+
+        parameters, mapping = merge_signatures(f, g)
+        # parameters is ('x', 'y', 'z', 'p')
+        # mapping is ((0, 1, 2), (0, 3))
+
+    Parameters
+    ----------
+    callable : callable
+        Callable whose parameters can be extracted with :func:`describe`.
+
+    Returns
+    -------
+    tuple(parameters, mapping)
+        parameters is the tuple of the merged parameter names.
+        mapping contains the mapping of parameters indices from the merged signature to
+        the original signatures.
+    """
+    args = []
+    mapping = []
+
+    for f in callables:
+        map = []
+        for i, k in enumerate(describe(f)):
+            if k in args:
+                map.append(args.index(k))
+            else:
+                map.append(len(args))
+                args.append(k)
+        mapping.append(tuple(map))
+
+    return args, mapping
 
 
 def describe(callable):
-    """Attempt to extract the function argument names.
+    """
+    Attempt to extract the function argument names.
 
-    **Returns**
+    Parameters
+    ----------
+    callable : callable
+        Callable whose parameters should be extracted.
 
-    Returns a list of strings with the parameters names if successful. If unsuccessful,
-    it returns an empty list.
-
-    **Function signature extraction algorithm**
+    Returns
+    -------
+    list
+        Returns a list of strings with the parameters names if successful and an empty
+        list otherwise.
 
+    Notes
+    -----
     Parameter names are extracted with the following three methods, which are attempted
     in order. The first to succeed determines the result.
 
     1. Using ``obj.func_code``. If an objects has a ``func_code`` attribute, it is used
        to detect the parameters. Examples::
 
-           def f(x, y):
+           def f(*args): # no signature
+               x, y = args
                return (x - 2) ** 2 + (y - 3) ** 2
 
-           f = lambda x, y: (x - 2) ** 2 + (y - 3) ** 2
+           f.func_code = make_func_code(("x", "y"))
 
        Users are encouraged to use this mechanism to provide signatures for objects that
        otherwise would not have a detectable signature. The function
@@ -702,9 +1014,8 @@ def describe(callable):
        names. This requires that a docstring is present which follows the Python
        standard formatting for function signatures.
 
-    **Handling of ambiguous cases**
-
-    How functions with positional and keyword argument are handled::
+    Ambiguous cases with positional and keyword argument are handled in the following
+    way::
 
         # describe returns [a, b];
         # *args and **kwargs are ignored
@@ -713,7 +1024,6 @@ def describe(callable):
         # describe returns [a, b, c];
         # positional arguments with default values are detected
         def fcn(a, b, c=1): ...
-
     """
     if _address_of_cfunc(callable) != 0:
         return ()
@@ -728,8 +1038,8 @@ def _arguments_from_func_code(obj):
     # Check (faked) f.func_code; for backward-compatibility with iminuit-1.x
     if hasattr(obj, "func_code"):
         fc = obj.func_code
-        return tuple(fc.co_varnames[: fc.co_argcount])
-    return ()
+        return list(fc.co_varnames[: fc.co_argcount])
+    return []
 
 
 def _arguments_from_inspect(obj):
@@ -737,7 +1047,7 @@ def _arguments_from_inspect(obj):
         # fails for builtin on Windows and OSX in Python 3.6
         signature = inspect.signature(obj)
     except ValueError:
-        return ()
+        return []
 
     args = []
     for name, par in signature.parameters.items():
@@ -748,26 +1058,31 @@ def _arguments_from_inspect(obj):
         if par.kind is inspect.Parameter.VAR_KEYWORD:
             break
         args.append(name)
-    return tuple(args)
+    return args
 
 
 def _arguments_from_docstring(obj):
     doc = inspect.getdoc(obj)
 
     if doc is None:
-        return ()
+        return []
 
     # Examples of strings we want to parse:
     #   min(iterable, *[, default=obj, key=func]) -> value
     #   min(arg1, arg2, *args, *[, key=func]) -> value
     #   Foo.bar(self, int ncall_me =10000, [resume=True, int nsplit=1])
 
-    name = obj.__name__
+    try:
+        # function wrapper functools.partial does not offer __name__,
+        # we cannot extract the signature in this case
+        name = obj.__name__
+    except AttributeError:
+        return []
 
     token = name + "("
     start = doc.find(token)
     if start < 0:
-        return ()
+        return []
     start += len(token)
 
     nbrace = 1
@@ -812,7 +1127,7 @@ def _arguments_from_docstring(obj):
     #   "arg1", "arg2", "key"
     #   "ncall_me", "resume", "nsplit"
 
-    return tuple(items)
+    return items
 
 
 def _guess_initial_step(val):
@@ -820,14 +1135,20 @@ def _guess_initial_step(val):
 
 
 def _key2index(var2pos, key):
-    if isinstance(key, int):
-        i = key
-        if i < 0:
-            i += len(var2pos)
-        if i < 0 or i >= len(var2pos):
-            raise IndexError
-    else:
-        i = var2pos[key]
+    if isinstance(key, slice):
+        start = var2pos[key.start] if isinstance(key.start, str) else key.start
+        stop = var2pos[key.stop] if isinstance(key.stop, str) else key.stop
+        start, stop, step = slice(start, stop, key.step).indices(len(var2pos))
+        return list(range(start, stop, step))
+    if isinstance(key, str):
+        return var2pos[key]
+    if isinstance(key, Sequence):
+        return [_key2index(var2pos, k) for k in key]
+    i = key
+    if i < 0:
+        i += len(var2pos)
+    if i < 0 or i >= len(var2pos):
+        raise IndexError
     return i
 
 
